set SSS_PROG "huygens"

############################# global variable are: ######################
# xy, color  - used for drawing in the lattice
# num_graphs - used for the LASSPgraph routines
# i,j - counting variables


set xy {0 0}
set color 1

set num_graphs 0

########################### Global variables linked to C ####################
# graphtype - intensity or amplitude

set graphtype intens		;# Intensity, amplitude

set Lambda 0.7
set Delta 0.
set AmplitudePerArea 1.		;# Amplitude of Incoming Wave
set AmplitudePerPointSource 1.	;# Amplitude per point if a=0
set X_Screen 100.0			;# Distance to Screen
set saturation_value 0.25		;# Color saturation
set DY_Screen [expr 1.5*$X_Screen]	;# Size of Screen
set L 10.

set N 100
set N_Screen 500

set N_slits 2
set N_points_per_wavelength 2	;# Integration points per meter width of slit
				;# (Gaussian quadrature)
set a 0.			;# Width of slit: point a=0 -> no integration
set d 6.			;# Width of array of slits 

set NMAX 1000

################################## center controls  ############
frame .g
frame .l
frame .g.ctop	;# Controls under graph
frame .g.cmid	;# Controls under graph
frame .g.cbot	;# Controls under graph
frame .l.c	;# Controls under lattice

LASSPscale2 .lambda -text "Lambda (m): " -scrollmin 0 -scrollmax 10  \
	-initial $Lambda -scrollinc .1 -type real -var Lambda \
	-command {huygens_sourcepick; huygens_display}
	
LASSPscale2 .delta -text "Phase (rad): " -scrollmin -3.1416 \
	-scrollmax 3.1416  -initial $Delta -scrollinc .1 -type real \
	-var Delta -command huygens_display

LASSPscale2 .d -text "d (m): " -scrollmin 0 \
	-scrollmax 10  -initial $d -scrollinc .1 -type real -var d \
	-command {huygens_sourcepick; huygens_display} \
	-test {%v>=0.0}

LASSPscale2 .a -text "a (m): " -scrollmin 0 \
	-scrollmax 10  -initial $a -scrollinc .1 -type real -var a \
	-command {huygens_sourcepick; huygens_display} \
	-test {%v>=0.0}

frame .n_slits
label .n_slits.l -text "Number of slits:"
LASSPentry .n_slits.e \
    {expr {[catch {set N_slits [%W get]}] \
    || [catch huygens_sourcepick] \
    || [catch huygens_display] }} -width 5
pack .n_slits.l  -side left
pack .n_slits.e  -side left -fill x -expand yes
LASSPentry_set .n_slits.e $N_slits
 
LASSPpresets .presets presetsave
button .configure -text "configure..." -command {configure}
button .help -text "help..." \
	-command {LASSPhelp2 .t2 [file join $SSS_DIR lib/huygens/huygens.htm]}
button .quit -text quit -command {exit}

frame .x_screen
label .x_screen.l -text "X Screen (m):"
LASSPentry .x_screen.e  \
    {expr {[catch {set X_Screen [%W get]}] \
    || [catch huygens_display] }} -width 5
pack .x_screen.l  -side left
pack .x_screen.e  -side left -fill x -expand yes
pack .x_screen -in .g.ctop -fill x -side left -expand yes -pady 1
LASSPentry_set .x_screen.e $X_Screen

frame .dy_screen
label .dy_screen.l -text "Screen size (m):"
LASSPentry .dy_screen.e  \
	{expr {[catch {set DY_Screen [%W get]}] \
	|| [catch huygens_display] }} -width 5
pack .dy_screen.l  -side left
pack .dy_screen.e  -side left -fill x -expand yes
pack .dy_screen -in .g.ctop -side left -fill x -expand yes -pady 1
LASSPentry_set .dy_screen.e $DY_Screen

pack .lambda -in .l.c -fill x -pady 2
pack .delta -in .l.c -fill x -pady 2
pack .d -in .l.c -fill x -pady 2
pack .a -in .l.c -fill x -pady 2
pack .n_slits -in .g.cbot -side right -fill x -pady 2
pack .presets -in .g.cbot -side right -fill x 
pack .configure -in .g.cmid -side right -fill x 
pack .help -in .g.cmid -side right -fill x 
pack .quit -in .g.cmid -side right -fill x 
wm minsize . 200 200

################# graph controls ##############################################
frame .gmenu
button .copygraph -text "copy graph" -command {
    incr num_graphs 1
    graph_create .graph$num_graphs .graph 
    .graph$num_graphs.graph configure -title $user_name
# JPS modified?
    .graph$num_graphs.graph psconfigure -landscape true \
	-pageheight 9i -pagewidth 6.5i
    .graph$num_graphs.graph postscript configure -landscape yes
}
LASSPselection .graphtype {} graphtype huygens_display "intens" "amplit" 
set user_name "Waldo"
frame .user_name
    label .user_name.l -text "Your Name:"
    LASSPentry .user_name.e  {catch {set user_name [%W get]}} -width 8
    LASSPentry_set .user_name.e $user_name
    pack .user_name.l -side left
    pack .user_name.e -side left -fill x -expand yes


pack .copygraph -in .gmenu -side left -padx 2
pack .graphtype -in .gmenu -side left -padx 2
pack .user_name -in .gmenu -side left -fill x -expand yes -padx 2
pack .gmenu -in .g -side top -fill x -pady 1

# XXX? frame .lmenu
# pack .lmenu -in .l -side top -fill x -pady 1

################### the main widgets ######################################
lattice .lattice -nx $N -ny $N -width 204 -height 204 

# ### these bindings allow one to draw in the lattice with the mouse button
# bind .lattice <Button-1> {         
#     set xy [.lattice locate %x %y]
#       if {0!=[string compare $xy "points not in lattice"]} {
#	set color [expr 1 - [.lattice flip [lindex $xy 0] [lindex $xy 1]]]
#	.lattice flip [lindex $xy 0] [lindex $xy 1] $color
# # XXX;	huygens_flipspin [lindex $xy 0] [lindex $xy 1] $color
#     }
# }
# bind .lattice <B1-Motion> {
#     set xy [.lattice locate %x %y]
#     if {0!=[string compare $xy "points not in lattice"]} {
# 	.lattice flip [lindex $xy 0] [lindex $xy 1] $color
# # XXX	huygens_flipspin [lindex $xy 0] [lindex $xy 1] $color
#     }
# }

blt_graph .graph -width 200 -height 200 -relief raised -bd 2 \
	-plotbackground white

.graph element create data
.graph element configure data -fg blue -linewidth 3
.graph legend configure -mapped no

# JPS modified back
## defaults to showing symbol, must set to null
#.graph element configure data -color blue -linewidth 3 -symbol ""
## newer BLT uses activeLine pen when a dataset is active
## need to change color and remove symbol
#.graph pen configure "activeLine" -color pink -symbol ""
.graph legend configure -mapped no
## mapped (shown) goes to hide in later blt
#.graph legend configure -hide true

.graph xaxis configure -title "Intensity vs. Y at screen"
.graph yaxis configure -title ""
LASSPgrzoom .graph
LASSPgrcursor .graph "format {%.6lf,   %.6lf} "

pack .graph -in .g -side top -fill both -expand yes 
# frame .latticeframe
# label .latticeframe.y -text "Y"
# pack .latticeframe.y -side left
# pack .lattice -in .latticeframe -side right -fill both -expand yes
# label .latticeframe.x -text "X"
# pack .latticeframe.x -side bottom
# pack .latticeframe -in .l -side top -fill both -expand yes
pack .lattice -in .l -side top -fill both -expand yes
pack .l.c -side right -fill x -expand yes -padx 5 -pady 5
pack .g.ctop -side top 
pack .g.cmid -side top 
pack .g.cbot -side top 
# pack .g -side right 
# pack .g.ctop -side top -fill x -expand yes -padx 1 -pady 1
# pack .g.cbot -side top -fill x -expand yes -padx 1 -pady 1
pack .g -side right -fill both -expand yes -padx 5 -pady 5
pack .l -side right -fill both -expand yes -padx 5 -pady 5
wm minsize . 200 200


################################## the presetsave procedure ############
proc presetsave {file} {
    #global variables
    foreach s {saturation_value L N N_Screen X_Screen DY_Screen N_slits \
		N_points_per_wavelength AmplitudePerArea \
		AmplitudePerPointSource} {
        global $s
        puts $file "set $s [set $s]"
    }

    #entries
    global N_slits
    puts $file "LASSPentry_set .n_slits.e $N_slits"
    global X_Screen
    puts $file "LASSPentry_set .x_screen.e $X_Screen"
    global DY_Screen
    puts $file "LASSPentry_set .dy_screen.e $DY_Screen"

    # allocation must be before huygens_display;
    # which is called automatically with some settings
    puts $file "huygens_allocate"
    puts $file {.lattice configure -nx $N -ny $N}

    #selections
    foreach s {.graphtype} {
        set val [LASSPselection_set $s]
        puts $file "LASSPselection_set $s \"$val\""
    }

    #sliders
    foreach s {.delta .lambda .d .a} {
        set val [LASSPscale2_configure $s -scrollinc]
        puts $file "LASSPscale2_configure $s -scrollinc $val"
        set val [LASSPscale2_configure $s -scrollmin]
        puts $file "LASSPscale2_configure $s -scrollmin $val"
        set val [LASSPscale2_configure $s -scrollmax]
        puts $file "LASSPscale2_configure $s -scrollmax $val"
        set val [LASSPscale2_configure $s -scrolltype]
        puts $file "LASSPscale2_configure $s -scrolltype $val"

	set val [LASSPscale2_get $s]
        puts $file "LASSPscale2_set -nocommand $s $val"
    } 


    #lattice
    puts $file ".lattice configure -color0 [lindex [.lattice configure -color0] 4]"
    puts $file ".lattice configure -color1 [lindex [.lattice configure -color1] 4]"
    puts $file ".lattice configure -color2 [lindex [.lattice configure -color2] 4]"
    puts $file ".lattice configure -color3 [lindex [.lattice configure -color3] 4]"
    puts $file ".lattice configure -color4 [lindex [.lattice configure -color4] 4]"
    puts $file ".lattice configure -color5 [lindex [.lattice configure -color5] 4]"
    puts $file ".lattice configure -color6 [lindex [.lattice configure -color6] 4]"
    puts $file ".lattice configure -color7 [lindex [.lattice configure -color7] 4]"
    puts $file ".lattice configure -color8 [lindex [.lattice configure -color8] 4]"
    puts $file ".lattice configure -color9 [lindex [.lattice configure -color9] 4]"

;#    set lattice [.lattice load]
;#    puts $file "set lattice $lattice"

    puts $file "huygens_sourcepick"
    puts $file "huygens_display"

}

################################## create a configure window ############
# configure N, N_Screen, L, X_Screen, saturation_value, DY_Screen, colors
proc configure {} {
    set w .huygensconfig

    if {[winfo exists $w]} {
	set list [winfo children $w]
	set index [lsearch $list $w.controls]
	set list [lreplace $list $index $index]
	foreach win $list  {
	    destroy $win
	}
	raise $w
    } else {
	toplevel $w
	wm title $w "huygens configure"
	frame $w.controls
	label $w.controls.l -text "huygens configure"
	button $w.controls.b -text close -command "destroy .huygensconfig"
	button $w.controls.b2 -text revert -command "configure"
	pack $w.controls.l -side left -fill x -expand yes
	pack $w.controls.b2 -side left -fill x -expand yes
	pack $w.controls.b -side left -fill x -expand yes
	pack $w.controls -fill x -expand yes -pady 1
    }

    frame $w.ell
    label $w.ell.l -text "Wavetank Size L (m):"
    LASSPentry $w.ell.e  \
	    {expr {[catch {set L [%W get]}] \
	    || [catch huygens_display] }}
    pack $w.ell.l  -side left
    pack $w.ell.e  -side left -fill x -expand yes
    pack $w.ell -fill x -expand yes -pady 1
    global L
    LASSPentry_set $w.ell.e $L

    frame $w.nx
    label $w.nx.l -text "Wavetank resolution:"
    LASSPentry $w.nx.e \
	    {expr {[catch {set N [%W get]}] \
	    || [catch huygens_allocate] \
	    || [catch {.lattice configure -nx [%W get]}] \
	    || [catch {.lattice configure -ny [%W get]}] \
	    || [catch huygens_display] }}
    pack $w.nx.l  -side left
    pack $w.nx.e  -side left -fill x -expand yes
    pack $w.nx -fill x -expand yes -pady 1
    global N
    LASSPentry_set $w.nx.e $N

    frame $w.amplitudePerArea
    label $w.amplitudePerArea.l -text "Wave Amplitude/Area:"
    LASSPentry $w.amplitudePerArea.e  \
	    {expr {[catch {set AmplitudePerArea [%W get]}] \
	    || [catch huygens_sourcepick] 
	    || [catch huygens_display] }}
    pack $w.amplitudePerArea.l  -side left
    pack $w.amplitudePerArea.e  -side left -fill x -expand yes
    pack $w.amplitudePerArea -fill x -expand yes -pady 1
    global AmplitudePerArea
    LASSPentry_set $w.amplitudePerArea.e $AmplitudePerArea

    frame $w.amplitudePerPointSource
    label $w.amplitudePerPointSource.l -text "Wave Amplitude/Slit a=0:"
    LASSPentry $w.amplitudePerPointSource.e  \
	    {expr {[catch {set AmplitudePerPointSource [%W get]}] \
	    || [catch huygens_sourcepick] 
	    || [catch huygens_display] }}
    pack $w.amplitudePerPointSource.l  -side left
    pack $w.amplitudePerPointSource.e  -side left -fill x -expand yes
    pack $w.amplitudePerPointSource -fill x -expand yes -pady 1
    global AmplitudePerPointSource
    LASSPentry_set $w.amplitudePerPointSource.e $AmplitudePerPointSource

    frame $w.n_screen
    label $w.n_screen.l -text "Data points on Screen:"
    LASSPentry $w.n_screen.e  \
	    {expr {[catch {expr -[%W get]}] \
	    || (!([%W get]>0)) \
	    || [catch {global NMAX}] || (!([%W get] < $NMAX)) \
	    || [catch {set N_Screen [%W get]}] \
	    || [catch huygens_allocate] \
	    || [catch huygens_display]  }}
    pack $w.n_screen.l  -side left
    pack $w.n_screen.e  -side left -fill x -expand yes
    pack $w.n_screen -fill x -expand yes -pady 1
    global N_Screen
    LASSPentry_set $w.n_screen.e $N_Screen

    frame $w.n_points_per_wavelength
    label $w.n_points_per_wavelength.l -text "Integration points:"
    LASSPentry $w.n_points_per_wavelength.e  \
	    {expr {[catch {set N_points_per_wavelength [%W get]}] \
	    || [catch huygens_sourcepick] \
	    || [catch huygens_display]  }}
    pack $w.n_points_per_wavelength.l  -side left
    pack $w.n_points_per_wavelength.e  -side left -fill x -expand yes
    pack $w.n_points_per_wavelength -fill x -expand yes -pady 1
    global N_points_per_wavelength
    LASSPentry_set $w.n_points_per_wavelength.e $N_points_per_wavelength
    
    frame $w.saturation
    label $w.saturation.l -text "Color Saturation Amp:"
    LASSPentry $w.saturation.e  \
	    {expr {[catch {set saturation_value [%W get]}] \
	    || [catch huygens_display] }}
    pack $w.saturation.l  -side left
    pack $w.saturation.e  -side left -fill x -expand yes
    pack $w.saturation -fill x -expand yes -pady 1
    global saturation_value
    LASSPentry_set $w.saturation.e $saturation_value

    frame $w.c
    frame $w.c.l	;# Right and left columns for colors
    frame $w.c.r

    frame $w.c.l.color0
    label $w.c.l.color0.l -text "color0:"
    LASSPentry $w.c.l.color0.e \
	    {catch {.lattice configure -color0 [%W get]}} -width 7
    pack $w.c.l.color0.l  -side left
    pack $w.c.l.color0.e  -side left -fill x -expand yes
    pack $w.c.l.color0 -fill x -expand yes -pady 1
    LASSPentry_set $w.c.l.color0.e [lindex [.lattice configure -color0] 4]

    frame $w.c.l.color1
    label $w.c.l.color1.l -text "color1:"
    LASSPentry $w.c.l.color1.e \
	    {catch {.lattice configure -color1 [%W get]}} -width 7
    pack $w.c.l.color1.l  -side left
    pack $w.c.l.color1.e  -side left -fill x -expand yes
    pack $w.c.l.color1 -fill x -expand yes -pady 1
    LASSPentry_set $w.c.l.color1.e [lindex [.lattice configure -color1] 4]

    frame $w.c.l.color2
    label $w.c.l.color2.l -text "color2:"
    LASSPentry $w.c.l.color2.e \
	    {catch {.lattice configure -color2 [%W get]}} -width 7
    pack $w.c.l.color2.l  -side left
    pack $w.c.l.color2.e  -side left -fill x -expand yes
    pack $w.c.l.color2 -fill x -expand yes -pady 1
    LASSPentry_set $w.c.l.color2.e [lindex [.lattice configure -color2] 4]

    frame $w.c.l.color3
    label $w.c.l.color3.l -text "color3:"
    LASSPentry $w.c.l.color3.e \
	    {catch {.lattice configure -color3 [%W get]}} -width 7
    pack $w.c.l.color3.l  -side left
    pack $w.c.l.color3.e  -side left -fill x -expand yes
    pack $w.c.l.color3 -fill x -expand yes -pady 1
    LASSPentry_set $w.c.l.color3.e [lindex [.lattice configure -color3] 4]

    frame $w.c.l.color4
    label $w.c.l.color4.l -text "color4:"
    LASSPentry $w.c.l.color4.e \
	    {catch {.lattice configure -color4 [%W get]}} -width 7
    pack $w.c.l.color4.l  -side left
    pack $w.c.l.color4.e  -side left -fill x -expand yes
    pack $w.c.l.color4 -fill x -expand yes -pady 1
    LASSPentry_set $w.c.l.color4.e [lindex [.lattice configure -color4] 4]

    frame $w.c.r.color5
    label $w.c.r.color5.l -text "color5:"
    LASSPentry $w.c.r.color5.e \
	    {catch {.lattice configure -color5 [%W get]}} -width 7
    pack $w.c.r.color5.l  -side left
    pack $w.c.r.color5.e  -side left -fill x -expand yes
    pack $w.c.r.color5 -fill x -expand yes -pady 1
    LASSPentry_set $w.c.r.color5.e [lindex [.lattice configure -color5] 4]

    frame $w.c.r.color6
    label $w.c.r.color6.l -text "color6:"
    LASSPentry $w.c.r.color6.e \
	    {catch {.lattice configure -color6 [%W get]}} -width 7
    pack $w.c.r.color6.l  -side left
    pack $w.c.r.color6.e  -side left -fill x -expand yes
    pack $w.c.r.color6 -fill x -expand yes -pady 1
    LASSPentry_set $w.c.r.color6.e [lindex [.lattice configure -color6] 4]

    frame $w.c.r.color7
    label $w.c.r.color7.l -text "color7:"
    LASSPentry $w.c.r.color7.e \
	    {catch {.lattice configure -color7 [%W get]}} -width 7
    pack $w.c.r.color7.l  -side left
    pack $w.c.r.color7.e  -side left -fill x -expand yes
    pack $w.c.r.color7 -fill x -expand yes -pady 1
    LASSPentry_set $w.c.r.color7.e [lindex [.lattice configure -color7] 4]

    frame $w.c.r.color8
    label $w.c.r.color8.l -text "color8:"
    LASSPentry $w.c.r.color8.e \
	    {catch {.lattice configure -color8 [%W get]}} -width 7
    pack $w.c.r.color8.l  -side left
    pack $w.c.r.color8.e  -side left -fill x -expand yes
    pack $w.c.r.color8 -fill x -expand yes -pady 1
    LASSPentry_set $w.c.r.color8.e [lindex [.lattice configure -color8] 4]

    frame $w.c.r.color9
    label $w.c.r.color9.l -text "color9:"
    LASSPentry $w.c.r.color9.e \
	    {catch {.lattice configure -color9 [%W get]}} -width 7
    pack $w.c.r.color9.l  -side left
    pack $w.c.r.color9.e  -side left -fill x -expand yes
    pack $w.c.r.color9 -fill x -expand yes -pady 1
    LASSPentry_set $w.c.r.color9.e [lindex [.lattice configure -color9] 4]

    pack $w.c.r -in $w.c -side right -expand yes
    pack $w.c.l -side right -expand yes
    pack $w.c -fill x -expand yes -pady 1

}

############################## special bindings #######################3
bind . <Key-Escape> {.quit invoke}
