set SSS_PROG "ising"

############################# global variable are: ######################
# xy, color  - used for drawing in the lattice
# num_graphs - used for the LASSPgraph routines
# sweepactive - whether or not a sweep is currently in progress
# sweepinc - does this # of sweeps between each display
# graphtype - Magnetization, Power Spectrum, or Autocorrelation?
# storedlattice - contains latest stored lattice 
# Mdata - store magnetization versus time
# Pdata - power spectrum
# Rdata - autocorrelation 
# i,j - counting variables


set xy {0 0}
set color 1

set num_graphs 0
set graphtype magnetization
set init_cond "T=inf"
set sweepactive 0
set T 2.4
set Hext 0
set sweepinc 1
set av_int 3

set storedlattice {}
# NX and NY must also be changed in ising.c for the initial call to lattice
set NX 40
set NY 40



################################## center controls  ############
frame .c

set sweepactive 0
button .sweep -text "run" -command {
    if {$sweepactive} {
	.sweep configure -text "run"
	set sweepactive 0
    } else {
	.sweep configure -text "stop"
	set sweepactive 1
	set temp $nsweeps
	for {set i $nsweeps} {$i>$sweepinc && $sweepactive} \
		{set i [expr $i-$sweepinc]} {
	    set averages [ising_sweep $T \
		    $Hext $sweepinc]
	    ising_display lattice
	    LASSPscale2_set .n [expr $i-$sweepinc]
	    update
	}
	if $sweepactive {
	    set averages [ising_sweep $T \
		    $Hext [expr round($i)]]
	}
	.gaverages1 configure -text [format "<M>=%lf     <(M-<M>)^2>=%lf" \
		[lindex $averages 0] [lindex $averages 1] ]
	.gaverages2 configure -text [format "<E>=%lf     <(E-<E>)^2>=%lf" \
		[lindex $averages 2] [lindex $averages 3] ]
	ising_display both
	.sweep configure -text "run" -rel raised
	set sweepactive 0
	LASSPscale2_set .n $temp
    }
}

LASSPscale2 .t -text "temperature (J): " -scrollmin 0 -scrollmax 10  \
	-initial 2.4 -scrollinc .1 -type real -var T
LASSPscale2 .h -text "external field (J): " -scrollmin -5 -scrollmax 5  \
	-initial 0 -scrollinc .1 -type real -var Hext
LASSPscale2 .n -text "number of sweeps: " -scrollmin 1 -scrollmax 10000  \
	-initial 1000 -scrollinc 1 -type real -var nsweeps  \
	-scrolltype log -test {%v>0}
LASSPscale2 .speed -text "speed: " -scrollmin 0 -scrollmax 100  \
	-initial 1 -scrollinc 1 -type integer -var sweepinc -test {%v>=1}


LASSPpresets .presets presetsave
button .configure -text "configure..." -command {configure}
button .help -text "help..." -command {LASSPhelp2 .t2 [file join $SSS_DIR lib/ising/ising.htm]}
button .quit -text quit -command {exit}

pack .sweep -in .c -fill x -pady 2
pack .t -in .c -fill x -pady 2
pack .h -in .c -fill x -pady 2
pack .n -in .c -fill x -pady 2
pack .speed -in .c -fill x -pady 2
pack .presets -in .c -fill x 
pack .configure -in .c -fill x 
pack .help -in .c -fill x 
pack .quit -in .c -fill x 
wm minsize . 200 200

################# graph controls ##############################################
frame .g
frame .gmenu
label .gaverages1
label .gaverages2
button .resetdata -text "reset" -command {
    .gaverages1 configure -text {}
    .gaverages2 configure -text {}
    ising_reset data 
    ising_display data 
}
button .copygraph -text "copy graph" -command {
    incr num_graphs 1
    graph_create .graph$num_graphs .graph 
}
LASSPselection .graphtype {} graphtype {ising_display data} \
	"magnetization" "power spectrum" "autocorrelation"

pack .resetdata -in .gmenu -side left -padx 2
pack .copygraph -in .gmenu -side left -padx 2
pack .graphtype -in .gmenu -side left -padx 2
pack .gmenu -in .g -side top -fill x -pady 1
pack .gaverages1 .gaverages2 -in .g -side top -fill x
################### lattice controls ########################################
frame .l
frame .lmenu
LASSPselection .init_cond {initial conditions} init_cond {set_init} \
	"T=inf" "M=1" "recall" "store"

proc set_init {} {
    global init_cond 

    if {$init_cond == "T=inf"} {
	ising_reset lattice 
	ising_display lattice 
    } elseif {$init_cond == "M=1"} {
	set NX [lindex [.lattice configure -nx] 4]
	set NY [lindex [.lattice configure -ny] 4]
	for {set j 0} {$j<$NY} {incr j} {
	    for {set i 0} {$i<$NX} {incr i} {
		ising_flipspin $i $j 1
	    }
	}
	ising_display lattice
    } elseif {$init_cond == "recall"} {
	global storedlattice
	set NX [lindex [.lattice configure -nx] 4]
	set NY [lindex [.lattice configure -ny] 4]
	for {set j 0} {$j<$NY} {incr j} {
	    for {set i 0} {$i<$NX} {incr i} {
		ising_flipspin $i $j [string index $storedlattice [expr $i+$NX*$j]]
	    }
	}
	ising_display lattice
    } elseif {$init_cond == "store"} {
	global storedlattice
	set storedlattice [.lattice load]
    }
}

button .init -text "init." -command {set_init}

pack .init_cond .init -in .lmenu -side left -padx 2
pack .lmenu -in .l -side top -fill x

################### the main widgets ######################################
lattice .lattice -nx 40 -ny 40 -width 300 -height 300 \
	-upColor red -downColor white

### these bindings allow one to draw in the lattice with the mouse button
bind .lattice <Button-1> {         
    set xy [.lattice locate %x %y]
    if {0!=[string compare $xy "points not in lattice"]} {
	set color [expr 1 - [.lattice flip [lindex $xy 0] [lindex $xy 1]]]
	.lattice flip [lindex $xy 0] [lindex $xy 1] $color
	ising_flipspin [lindex $xy 0] [lindex $xy 1] $color
    }
}
bind .lattice <B1-Motion> {
    set xy [.lattice locate %x %y]
    if {0!=[string compare $xy "points not in lattice"]} {
	.lattice flip [lindex $xy 0] [lindex $xy 1] $color
	ising_flipspin [lindex $xy 0] [lindex $xy 1] $color
    }
}
blt_graph .graph -width 300 -height 300 -relief raised -bd 2 \
	-plotbackground white
.graph element create data
.graph element configure data -fg blue
.graph legend configure -mapped no
.graph yaxis configure -title magnetization
.graph xaxis configure -title "t (# of sweeps)"
LASSPgrzoom .graph
LASSPgrcursor .graph "format {%.4lf,   %.4lf} "

pack .graph -in .g -side top -fill both -expand yes 
pack .lattice -in .l -side bottom -fill both -expand no
pack .g -side right -fill both -expand yes -padx 5 -pady 5
pack .c -side right -fill y -padx 5 -pady 5
pack .l -side right -fill both -expand yes -padx 5 -pady 5
wm minsize . 200 200


################################## the presetsave procedure ############
proc presetsave {file} {
    #global variables
    foreach s {} {
        global $s
        puts $file "set $s [set $s]"
    }
    global storedlattice
    if {[string length $storedlattice] != 0} {
	puts $file "set storedlattice $storedlattice"
    }

    #selections
    foreach s {.graphtype .init_cond} {
        set val [LASSPselection_set $s]
        puts $file "LASSPselection_set $s \"$val\""
    }

    #sliders
    foreach s {.t .h .n .speed} {
        set val [LASSPscale2_configure $s -scrollinc]
        puts $file "LASSPscale2_configure $s -scrollinc $val"
        set val [LASSPscale2_configure $s -scrollmin]
        puts $file "LASSPscale2_configure $s -scrollmin $val"
        set val [LASSPscale2_configure $s -scrollmax]
        puts $file "LASSPscale2_configure $s -scrollmax $val"
        set val [LASSPscale2_configure $s -scrolltype]
        puts $file "LASSPscale2_configure $s -scrolltype $val"
# rbr no fonts in presets
#        set val [LASSPscale2_configure $s -font]
#        puts $file "LASSPscale2_configure $s -font $val"

	set val [LASSPscale2_get $s]
        puts $file "LASSPscale2_set -nocommand $s $val"
    } 
    
    
    #lattice
    puts $file ".lattice configure -upColor [lindex [.lattice configure -upColor] 4]"
    puts $file ".lattice configure -downColor [lindex [.lattice configure -downColor] 4]"
    set nx [lindex [.lattice configure -nx] 4]
    set ny [lindex [.lattice configure -ny] 4]
    puts $file ".lattice configure -nx $nx -ny $ny"
    puts $file "ising_configure -nx $nx -ny $ny"

    set lattice [.lattice load]
    puts $file "set lattice $lattice"

    puts $file "set nx [lindex [.lattice configure -nx] 4]"
    puts $file "set ny [lindex [.lattice configure -ny] 4]"
    puts $file {
	for {set j 0} {$j<$ny} {incr j} {
	    for {set i 0} {$i<$nx} {incr i} {
		ising_flipspin $i $j [string index $lattice [expr $i+$nx*$j]]
	    }
	}
    }
    puts $file "ising_display lattice"

    #graph
    puts $file ".resetdata invoke"
}

################################## create a configure window ############
# configure NX, NY, colors, font, displayspeed, sweepincrement
proc configure {} {
    set w .isingconfig

    if {[winfo exists $w]} {
	set list [winfo children $w]
	set index [lsearch $list $w.controls]
	set list [lreplace $list $index $index]
	foreach win $list  {
	    destroy $win
	}
	raise $w
    } else {
	toplevel $w
	wm title $w "ising configure"
	frame $w.controls
	label $w.controls.l -text "ising configure"
	button $w.controls.b -text close -command "destroy .isingconfig"
	button $w.controls.b2 -text revert -command "configure"
	pack $w.controls.l -side left -fill x -expand yes
	pack $w.controls.b2 -side left -fill x -expand yes
	pack $w.controls.b -side left -fill x -expand yes
	pack $w.controls -fill x -expand yes -pady 1
    }    

    frame $w.nx
    label $w.nx.l -text "lattice width:"
    LASSPentry $w.nx.e \
	    {expr {[catch {ising_configure -nx [%W get]}] \
	    || [catch {.lattice configure -nx [%W get]}] \
	    || [catch {ising_display both}] }}
    pack $w.nx.l  -side left
    pack $w.nx.e  -side left -fill x -expand yes
    pack $w.nx -fill x -expand yes -pady 1
    LASSPentry_set $w.nx.e [lindex [.lattice configure -nx] 4]
    
    frame $w.ny
    label $w.ny.l -text "lattice height:"
    LASSPentry $w.ny.e  \
	    {expr {[catch {ising_configure -ny [%W get]}] \
	    || [catch {.lattice configure -ny [%W get]}] \
	    || [catch {ising_display both}]  }}
    pack $w.ny.l  -side left
    pack $w.ny.e  -side left -fill x -expand yes
    pack $w.ny -fill x -expand yes -pady 1
    LASSPentry_set $w.ny.e [lindex [.lattice configure -ny] 4]
    
    frame $w.upcolor
    label $w.upcolor.l -text "spin-up color:"
    LASSPentry $w.upcolor.e \
	    {catch {.lattice configure -upColor [%W get]}}
    pack $w.upcolor.l  -side left
    pack $w.upcolor.e  -side left -fill x -expand yes
    pack $w.upcolor -fill x -expand yes -pady 1
    LASSPentry_set $w.upcolor.e [lindex [.lattice configure -upColor] 4]

    frame $w.downcolor
    label $w.downcolor.l -text "spin-down color:"
    LASSPentry $w.downcolor.e \
	    {catch {.lattice configure -downColor [%W get]}}
    pack $w.downcolor.l  -side left
    pack $w.downcolor.e  -side left -fill x -expand yes
    pack $w.downcolor -fill x -expand yes -pady 1
    LASSPentry_set $w.downcolor.e [lindex [.lattice configure -downColor] 4]

    frame $w.av_int
    label $w.av_int.l -text "average length:"
    LASSPentry $w.av_int.e \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || ![expr 1.0*([%W get]/2) != 0.5*([%W get])] \
	    || [catch {set av_int [%W get]}] }}
    pack $w.av_int.l  -side left
    pack $w.av_int.e  -side left -fill x -expand yes
    pack $w.av_int -fill x -expand yes -pady 1
    global av_int
    LASSPentry_set $w.av_int.e $av_int
 
}

############################## special bindings #######################3
bind . <Key-space> {.sweep invoke}
bind . <Key-i> {.init invoke}
bind . <Key-Escape> {.quit invoke}
