set SSS_PROG "pythag"

# Debugging Tool
# LASSPinterp .interp

########################### Global variables linked to C ####################

# Parameters for Selection Menus

set r_bdy Fixed		;# Boundary conditions: Fixed, Free, Refless
set l_bdy Fixed		
set graph_vs_X Y	;# What to Graph as function of X
			;# Y, Y DOT, PE, KE, u, Power, or None
set graph_vs_T Y	;# What to Graph as function of time
set incoming_l Pulse	;# Pulse, Wave, Packet, NegPulse, or None at Left
set incoming_r None	;# Pulse, Wave, Packet, NegPulse, or None at Right

# Parameters for Configure Menu

set nsites 256		;# Steps along string
set L 10		;# Ten meter rope
set running 0
set y_mult 1		;# Allows manual scaling of vertical axis default



# Parameters for Sliders

set amplitude 0.05	;# One centimeter
set omega 300.		;# Frequency of wave or packet
set FWHM 0.04		;# Width of pulse or packet
set tmax 1.		;# Time to run
set tSim 0.		;# Time during run


set mu1 0.1		;# One kg/m: mass density of first piece of string
set mu2 0.025		;# One kg/m: mass density of second piece of string
set mu3 0.1		;# One kg/m: mass density of third piece of string

set x12 $L		;# Past end of rope to start
set x23 $L
set x_graph [expr 0.5*$L]  ;# Point to graph vs time
set tau 160		;# One hundred newtons (about 10 g times mass)

####################### Global variables Not linked to C ####################

set graph_time_skip 20	;# Skip between graph draws

set user_name "Waldo"

set num_graphs_vs_X 0
set num_graphs_vs_T 0

set NMAX 1000		;# Max data for y: must be consistent with pythag.c
set tData {}		;# Time data

proc epsilon_t_set {} {	;# Reset to keep algorithm barely stable
    global mu1 mu2 mu3 x12 x23
    global L tau nsites
    global epsilon_t

    set mu_min $mu1
    if { ( $x12 < $L ) && ( $mu2 < $mu_min ) } { set mu_min $mu2 }
    if { ( $x23 < $L ) && ( $mu3 < $mu_min ) } { set mu_min $mu3 }

    set c [expr sqrt($tau/$mu_min)]	;# Velocity of sound

;# Characteristic distance over characteristic time
;# Leapfrog goes unstable if epsilon_t gets much larger than this

    set epsilon_t [expr 0.5*($L/double($nsites))/$c]	

}

epsilon_t_set 		;# Time step for LeapFrog

set ymax 0		;# Scales for y axis (temporary variables)
set ymin 0

########## create & init main graph, set scales #############

frame .fx_plot
set w .fx_plot
frame $w.menus


LASSPselection .l_bdy "BC:" l_bdy {} \
	"Fixed" "Free" "Refless"
LASSPselection .incoming_l "Forcing:" incoming_l {reset_scales} \
	"Pulse" "Wave" "Packet" "NegPulse" "None"

LASSPselection .incoming_r "Forcing:" incoming_r {reset_scales} \
	"None" "Pulse" "Wave" "Packet" "NegPulse"
LASSPselection .r_bdy "BC:" r_bdy {} \
	"Fixed" "Free" "Refless"

pack .l_bdy -in $w.menus -side left -padx 2
pack .incoming_l -in $w.menus -side left -padx 2

pack .r_bdy -in $w.menus -side right -padx 2
pack .incoming_r -in $w.menus -side right -padx 2

pack $w.menus -side top -fill x -pady 1

# Gives proper y-axis maximum and minimum for various plotted quantities
proc yscale { graph_vs } {
    global amplitude ymax ymin FWHM tau mu1 mu2 mu3 x12 x23 L
    global incoming_l incoming_r omega y_mult

# y_mult allows manual rescaling of axes, changing scale of y
# abscissa scales as y_mult for Y and Y DOT
# abscissa scales as y_mult^2 for PE, KE, u, and Power

    set mu_max $mu1
    if { ( $x12 < $L ) && ( $mu2 > $mu_max ) } { set mu_max $mu2 }
    if { ( $x23 < $L ) && ( $mu3 > $mu_max ) } { set mu_max $mu3 }

# max value of derivative of gaussian of full width FWHM at half maximum

    set omega_max [expr 1.17741/$FWHM]

# If either side isn't a pulse, choose maximum of 1.17741/FWHM and omega
    if { ( $incoming_l == "Wave" ) || ( $incoming_l == "Packet" ) \
	 || ( $incoming_r == "Wave" ) || ( $incoming_r == "Packet" ) } {
	if { $omega_max < $omega } { set omega_max $omega }
    }
    if {$graph_vs == "Y"} {
	set ymax [expr 2 * $amplitude * $y_mult]
	set ymin [expr -1.2*$amplitude * $y_mult]
    } elseif { $graph_vs == "Y DOT" } {
	set ymax [expr 1.5*$amplitude*$omega_max*$y_mult]
	set ymin [expr -$ymax]
    } elseif { ( $graph_vs == "PE" ) || ( $graph_vs == "KE" ) \
			|| ( $graph_vs == "u" )} {
	set ymax [expr  1.5*$mu_max * $amplitude*$amplitude*$y_mult*$y_mult \
			* ($omega_max * $omega_max)]
	set ymin [expr -0.1*$ymax]
    } elseif { $graph_vs == "Power" } {
	set ymax [expr 1.5 * sqrt($mu_max * $tau) * $amplitude * $amplitude \
		* $y_mult * $y_mult * ($omega_max * $omega_max)]
	set ymin [expr -$ymax]
    } elseif { $graph_vs == "None" } {
    }

}

proc reset_scales {} {
    global graph_vs_X graph_vs_T ymin ymax .x_plot .t_plot L

    yscale $graph_vs_X
    .x_plot yaxis configure -min $ymin -max $ymax
    .x_plot xaxis configure -min 0 -max $L

    yscale $graph_vs_T
    .t_plot yaxis configure -min $ymin -max $ymax
    .t_plot xaxis configure -min {} -max {}
}
    

# JPS modified?
# blt_graph .x_plot -width 600 -height 200 -bd 2 \
#    -relief raised -plotbackground white



blt_graph .x_plot -width 600 -height 200 -bd 2 \
    -relief raised -plotbackground white

# Modified version of LASSPgrzoom, to reset_scale rather than auto-scale
# when double click

source pythgrto.tcl
PYTgrzoom .x_plot

LASSPgrcursor .x_plot "format {%.5lf, %.5lf} "
.x_plot element create data1
.x_plot element create data2
.x_plot element create data3
.x_plot element create dataSpot
# JPS: modified?
# .x_plot element configure dataSpot -fg black -linewidth 8
# defaults to showing symbol, must set to null
#.x_plot element configure dataSpot -color black -linewidth 8 -symbol ""
# newer BLT uses activeLine pen when a dataset is active
# need to change color and remove symbol
#.x_plot pen configure "activeLine" -color pink -symbol ""
.x_plot element configure dataSpot -fg black -linewidth 8

# Thickness of line for data1, data2, and data3 
# proportional to square root of mass density

# Define variables for thickness and colors for mu1, mu2, and mu3
set mu1_initial $mu1
set thick_initial 3
set thick1 3
set thick2 3
set thick3 3

set color1 blue
set color2 red
set color3 purple



# As a procedure, so can change as muX changed in configure menu
proc set_thickness {} {
    global mu1_initial thick_initial mu1 mu2 mu3 thick1 thick2 thick3
    global .x_plot color1 color2 color3

    set thick1 [expr floor($thick_initial * sqrt($mu1/$mu1_initial) + 0.5)]
    set thick2 [expr floor($thick_initial * sqrt($mu2/$mu1_initial) + 0.5)]
    set thick3 [expr floor($thick_initial * sqrt($mu3/$mu1_initial) + 0.5)]

# JPS: modified?
#    .x_plot element configure data1 -fg $color1 -linewidth $thick1
#    .x_plot element configure data2 -fg $color2 -linewidth $thick2
#    .x_plot element configure data3 -fg $color3 -linewidth $thick3
# defaults to showing symbol, must be set to null
#    .x_plot element configure data1 -color $color1 -linewidth $thick1 -symbol ""
#    .x_plot element configure data2 -color $color2 -linewidth $thick2 -symbol ""
#    .x_plot element configure data3 -color $color3 -linewidth $thick3 -symbol ""
# JPS: don't know whether need to configure pen again? 
#    .x_plot pen configure "activeLine" -color pink -symbol ""
.x_plot element configure data1 -fg $color1 -linewidth $thick1
.x_plot element configure data2 -fg $color2 -linewidth $thick2}
.x_plot element configure data3 -fg $color3 -linewidth $thick3
set_thickness

# JPS modified?
# .x_plot legend configure -mapped no
# mapped (shown) goes to hide in later blt
#.x_plot legend configure -hide true
.x_plot legend configure -mapped no

.x_plot xaxis configure -title "$graph_vs_X vs. X (m)"
.x_plot yaxis configure -title {}
.x_plot xaxis configure -min 0 -max $L

yscale $graph_vs_X
.x_plot yaxis configure -min $ymin -max $ymax

pack .x_plot -in $w -side top -fill both -expand yes

pack $w -side top -fill both -expand yes


##########  create & init small graph, set scales #############

frame .ft_plot
set w .ft_plot
frame $w.menus
button .t_copygraph -text "Copy \n Graph" -command {
    incr num_graphs_vs_T 1
    graph_create .t_plot$num_graphs_vs_T .t_plot
    .t_plot$num_graphs_vs_T.graph configure -title $user_name
# JPS modified?
#    .t_plot$num_graphs_vs_T.graph psconfigure -landscape true \
#        -pageheight 9i -pagewidth 6.5i                          
#    .t_plot$num_graphs_vs_T.graph postscript configure -landscape yes
.t_plot$num_graphs_vs_T.graph psconfigure -landscape true \
    -pageheight 9i -pagewidth 6.5i
}

frame .t_graphtype_box -bd 2 -relief raised
LASSPselection .t_graphtype {} graph_vs_T \
	{
	    .t_plot xaxis configure -title "$graph_vs_T vs. T (s)"
	    reset_scales
	    graph_vs_T_initialize
	} \
	"Y" "Y DOT" "PE" "KE" "u" "Power" "None"
label .t_graphtype_label -text "vs. time"
pack .t_graphtype -in .t_graphtype_box 
pack .t_graphtype_label -in .t_graphtype_box -padx 2

pack .t_graphtype_box -in $w.menus -padx 2
pack .t_copygraph -in $w.menus -padx 2
pack $w.menus -in $w -side right -fill x -pady 1

# JPS modified?
#blt_graph .t_plot -width 200 -height 200 -bd 2 \
#    -relief raised -plotbackground white

blt_graph .t_plot -width 200 -height 200 -bd 2 \
    -relief raised -plotbackground white
PYTgrzoom .t_plot
LASSPgrcursor .t_plot "format {%.5lf, %.5lf} "
.t_plot element create data
# JPS: modified?
# .t_plot element configure data -fg blue -linewidth 3
# defaults to showing symbol, must set to null
#.t_plot element configure data -color blue -linewidth 3 -symbol ""
.t_plot element configure data -fg blue -linewidth 3
# newer BLT uses activeLine pen when a dataset is active
# need to change color and remove symbol
#.t_plot pen configure "activeLine" -color pink -symbol ""
# .t_plot legend configure -mapped no
# mapped (shown) goes to hide in later blt
#.t_plot legend configure -hide true
.t_plot legend configure -mapped no
.t_plot xaxis configure -title "$graph_vs_T vs. T (s)"
.t_plot yaxis configure -title {}

reset_scales

pack .t_plot -in $w -side left -fill both -expand yes

pack $w -side left -fill both -expand yes


################ initialize graphs #################

proc graph_initialize {} {
    global tSim  tData

    stop

    # Reset graph to no data
    for {set i 1} {$i<3} {incr i 1} {
	.x_plot element configure data$i -data {}
    }
    .x_plot element configure dataSpot -data {}

# JPS DEBUG?
    set tData {}
    .t_plot element configure data -data $tData

    set tSim 0.
    LASSPentry_set .tSim $tSim

    # Draw the initial stuff
    pythag_initialize

}



################# the presetsave procedure ############################
proc presetsave {file} {

    #global variables
    foreach s {nsites graph_time_skip L epsilon_t mu1 mu2 mu3 x12 x23 x_graph tau \
	running tSim amplitude omega FWHM tmax thick1 thick2 thick3 \
	thick_initial mu1_initial \
	color1 color2 color3 y_mult} {
	global $s
	puts $file "set $s [set $s]"
    }

    #selections
    foreach s {.x_graphtype .t_graphtype \
	.r_bdy .l_bdy .incoming_l .incoming_r} {
	set val [LASSPselection_set $s]
	puts $file "LASSPselection_set $s $val"
    }


    #commands to execute

    puts $file "stop; set_thickness; .initialize invoke; .run invoke"

}


##################### create a configure window ###################
# configure natoms
proc configure {} {
   if {[winfo exists .pythagconfig]} {
	foreach win [winfo children .pythagconfig] {
	    destroy $win
	}
   } else {
	toplevel .pythagconfig
	wm title .pythagconfig "pythag configure"
   }
   set w .pythagconfig
    
   raise .pythagconfig
   frame $w.controls
   label $w.controls.l -text "pythag configure"
   button $w.controls.b -text close -command "destroy .pythagconfig"
   button $w.controls.b2 -text revert -command "configure"
   pack $w.controls.l -side left -fill x -expand yes
   pack $w.controls.b2 -side left -fill x -expand yes
   pack $w.controls.b -side left -fill x -expand yes
   pack $w.controls -fill x -expand yes -pady 1

   frame $w.packet -bd 3 -relief raised
    label $w.packet.l -text "Pulse, Packet, or Wave Shape"
    pack $w.packet.l

    frame $w.packet.amplitude
    label $w.packet.amplitude.l -text "Amplitude (m)"
    LASSPentry $w.packet.amplitude.e \
	    {expr {[catch {expr -[%W get]}] \
	    || [catch {set amplitude [%W get]}] \
	    || [catch {graph_initialize}]}} -width 6
    pack $w.packet.amplitude.l -side left
    pack $w.packet.amplitude.e -side left -fill x -expand yes
    pack $w.packet.amplitude -side left -fill x -expand yes
    global amplitude
    LASSPentry_set $w.packet.amplitude.e $amplitude

    frame $w.packet.omega
    label $w.packet.omega.l -text "Omega (rad/s)"
    LASSPentry $w.packet.omega.e \
	    {expr {[catch {expr -[%W get]}] \
	    || [catch {set omega [%W get]}] \
	    || [catch {graph_initialize}]}} -width 6
    pack $w.packet.omega.l -side left
    pack $w.packet.omega.e -side left -fill x -expand yes
    pack $w.packet.omega -side left -fill x -expand yes
    global omega
    LASSPentry_set $w.packet.omega.e $omega

    frame $w.packet.fwhm
    label $w.packet.fwhm.l -text "FWHM (s)"
    LASSPentry $w.packet.fwhm.e \
	    {expr {[catch {expr -[%W get]}] \
	    || [catch {set FWHM [%W get]}] \
	    || [catch {graph_initialize}]}} -width 6
    pack $w.packet.fwhm.l -side left
    pack $w.packet.fwhm.e -side left -fill x -expand yes
    pack $w.packet.fwhm -side left -fill x -expand yes
    global FWHM
    LASSPentry_set $w.packet.fwhm.e $FWHM

   pack $w.packet -fill x -expand yes -pady 2

   frame $w.density -bd 3 -relief raised
    label $w.density.l -text "Mass Density Distribution mu(x)"
    pack $w.density.l

    frame $w.mu
    label $w.mu.l1 -text "Mu1 (kg/m):"
    LASSPentry $w.mu.e1 \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || [catch {set mu1 [%W get]}] \
	    || [catch {
		    set_thickness
		    epsilon_t_set		;# For numerical stability
		    graph_initialize
		 }]}} -width 6
    pack $w.mu.l1  -side left
    pack $w.mu.e1  -side left -fill x -expand yes
    label $w.mu.l2 -text "Mu2 (kg/m):"
    LASSPentry $w.mu.e2 \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || [catch {set mu2 [%W get]}] \
	    || [catch {
		    set_thickness
		    epsilon_t_set		;# For numerical stability
		    graph_initialize
		 }]}} -width 6
    pack $w.mu.l2  -side left
    pack $w.mu.e2  -side left -fill x -expand yes
    label $w.mu.l3 -text "Mu3 (kg/m):"
    LASSPentry $w.mu.e3 \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || [catch {set mu3 [%W get]}] \
	    || [catch {
		    set_thickness
		    epsilon_t_set		;# For numerical stability
		    graph_initialize
		 }]}} -width 6
    pack $w.mu.l3  -side left
    pack $w.mu.e3  -side left -fill x -expand yes
    global mu1 mu2 mu3
    LASSPentry_set $w.mu.e1 $mu1
    LASSPentry_set $w.mu.e2 $mu2
    LASSPentry_set $w.mu.e3 $mu3
    pack $w.mu -in $w.density -fill x -expand yes -pady 1

    frame $w.steps
    label $w.steps.l12 -text "Step X12 (m):"
    LASSPentry $w.steps.e12 \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || !([%W get]<=$x23) \
	    || !([%W get]<=$L) \
	    || [catch {set x12 [%W get]}] \
	    || [catch {
		    epsilon_t_set
		    graph_initialize}]}} -width 6
    pack $w.steps.l12  -side left
    pack $w.steps.e12  -side left -expand yes
    label $w.steps.l23 -text "Step X23 (m):"
    LASSPentry $w.steps.e23 \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || !([%W get]>=$x12) \
	    || !([%W get]<=$L) \
	    || [catch {set x23 [%W get]}] \
	    || [catch {
		    epsilon_t_set
		    graph_initialize}]}} -width 6
    pack $w.steps.l23  -side left
    pack $w.steps.e23  -side left -fill x -expand yes

    pack $w.steps -in $w.density -expand yes -pady 1
    global x12 x23
    LASSPentry_set $w.steps.e12 $x12
    LASSPentry_set $w.steps.e23 $x23

   pack $w.density -fill x -expand yes -pady 2


   frame $w.graphics -bd 3 -relief raised
    label $w.graphics.l -text "Graphics Controls"
    pack $w.graphics.l

    frame $w.x_graph
    label $w.x_graph.l -text "X for vs T (m):"
    LASSPentry $w.x_graph.e \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || [catch {set x_graph [%W get]}] \
	    || [catch {graph_initialize}]}} -width 6
    pack $w.x_graph.l  -side left
    pack $w.x_graph.e  -side left -fill x -expand yes
    pack $w.x_graph -in $w.graphics -side left -fill x -expand yes -pady 1
    global x_graph
    LASSPentry_set $w.x_graph.e $x_graph

    frame $w.graph_time_skip
    label $w.graph_time_skip.l -text "graph time skip:"
    LASSPentry $w.graph_time_skip.e \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || !([%W get]<32768) \
	    || [catch {set graph_time_skip [%W get]}] }} -width 6
    pack $w.graph_time_skip.l  -side left
    pack $w.graph_time_skip.e  -side left -fill x -expand yes
    pack $w.graph_time_skip -in $w.graphics \
		-side left -fill x -expand yes -pady 1
    global graph_time_skip
    LASSPentry_set $w.graph_time_skip.e $graph_time_skip

    frame $w.y_mult
    label $w.y_mult.l -text "Y mult:"
    LASSPentry $w.y_mult.e \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || [catch {set y_mult [%W get]}] \
	    || [catch {
		reset_scales}]}} -width 6
    pack $w.y_mult.l  -side left
    pack $w.y_mult.e  -side left -fill x -expand yes
    pack $w.y_mult -in $w.graphics -fill x -expand yes -pady 1
    global y_mult
    LASSPentry_set $w.y_mult.e $y_mult

   pack $w.graphics -fill x -expand yes -pady 2
    
   frame $w.string -bd 3 -relief raised
    label $w.string.l -text "String State"
    pack $w.string.l

    frame $w.tau
    label $w.tau.l -text "Tension Tau (N):"
    LASSPentry $w.tau.e \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || [catch {set tau [%W get]}] \
	    || [catch {
		    epsilon_t_set
		    graph_initialize}]}} -width 6
    pack $w.tau.l  -side left
    pack $w.tau.e  -side left -fill x -expand yes
    pack $w.tau -in $w.string -side left -fill x -expand yes -pady 1
    global tau
    LASSPentry_set $w.tau.e $tau
 
    frame $w.ell
    label $w.ell.l -text "Length L (m):"
    LASSPentry $w.ell.e \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || !([%W get]>=$x23) \
	    || [catch {set L [%W get]}] \
	    || [catch {
		    epsilon_t_set
		    reset_scales
		    graph_initialize}]}} -width 6
    pack $w.ell.l  -side left
    pack $w.ell.e  -side left -fill x -expand yes
    pack $w.ell -in $w.string -side left -fill x -expand yes -pady 1
    global L
    LASSPentry_set $w.ell.e $L
   pack $w.string -fill x -expand yes -pady 2

   frame $w.accuracy -bd 3 -relief raised
    label $w.accuracy.l -text "Numerical Accuracy Controls"
    pack $w.accuracy.l 

    frame $w.nsites
    label $w.nsites.l -text "# string pieces:"
    global NMAX
    LASSPentry $w.nsites.e \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || !([%W get]<$NMAX) \
	    || [catch {set nsites [%W get]}] \
	    || [catch {
		    epsilon_t_set
		    graph_initialize}]}} -width 6
    pack $w.nsites.l  -side left
    pack $w.nsites.e  -side left -fill x -expand yes
    pack $w.nsites -in $w.accuracy -side left -fill x -expand yes -pady 1
    global nsites
    LASSPentry_set $w.nsites.e $nsites

    frame $w.epsilon_t
    label $w.epsilon_t.l -text "epsilon_t (s) (auto-set):"
    LASSPentry $w.epsilon_t.e \
	    {expr {[catch {expr -[%W get]}] \
	    || !([%W get]>0) \
	    || [catch {set epsilon_t [%W get]}] \
	    || [catch {graph_initialize}]}} -width 6
    pack $w.epsilon_t.l  -side left
    pack $w.epsilon_t.e  -side left -fill x -expand yes
    pack $w.epsilon_t -in $w.accuracy -side left -fill x -expand yes -pady 1
    global epsilon_t
    LASSPentry_set $w.epsilon_t.e $epsilon_t

   pack $w.accuracy -fill x -expand yes -pady 2

}



################################# create controls  #################

frame .x_controls
button .x_copygraph -text "Copy Graph" -command {
    incr num_graphs_vs_X 1
    graph_create .x_plot$num_graphs_vs_X .x_plot
    .x_plot$num_graphs_vs_X.graph configure -title $user_name
# JPS modified?
#    .x_plot$num_graphs_vs_X.graph psconfigure -landscape true \
#        -pageheight 9i -pagewidth 6.5i
#    .x_plot$num_graphs_vs_X.graph postscript configure -landscape yes
    .x_plot$num_graphs_vs_X.graph psconfigure -landscape true \
        -pageheight 9i -pagewidth 6.5i
}

frame .x_graphtype_box -bd 2 -relief raised
LASSPselection .x_graphtype {} graph_vs_X \
	{
	    .x_plot xaxis configure -title "$graph_vs_X vs. X (m)"
	    reset_scales 
	    draw_graph_vs_X
	} \
	"Y" "Y DOT" "PE" "KE" "u" "Power" "None"
label .x_graphtype_label -text "vs. X"
pack .x_graphtype -in .x_graphtype_box -side left 
pack .x_graphtype_label -in .x_graphtype_box -side left -padx 10

pack .x_copygraph -in .x_controls -side right -padx 2
pack .x_graphtype_box -in .x_controls -side right -padx 2

pack .x_controls -side top

frame .c

proc stop {} {
    global tmax_store running .tmax
    if {$running} {
	.run configure -text "Run"
	LASSPentry_set .tmax $tmax_store
	set running 0
	update
    }
}

frame .topbuttons
button .run -text "Run" -width 10 -command {
    if {$running} {stop} else {
	.run configure -text "Stop"
	update
	set running 1
	set tmax_store $tmax
	set t_start $tSim
	set t_final [expr $t_start + $tmax]
	set no_graph_count 0

# Run forward in time or backward in time
	if {$tmax > 0} {
	  while {$tSim <= [expr $t_final - $epsilon_t] && $running} {
	    pythag_evolve $epsilon_t	;# Increments t_sim by epsilon_t
	    draw_graph_vs_T
	    incr no_graph_count 1
	    if {$no_graph_count>=$graph_time_skip} {
	        LASSPentry_set .tmax [expr $t_final - $tSim]
	        LASSPentry_set .tSim $tSim
	  	draw_graph_vs_X
		set no_graph_count 0
	  	update
	    }
	  }
	  if {($tSim < $t_final) && $running} {
	    pythag_evolve [expr $t_start + $tmax_store - $tSim] ;# Remainder
          }
	  draw_graph_vs_T
	  LASSPentry_set .tmax [expr $t_final - $tSim]
	  LASSPentry_set .tSim $tSim
	  draw_graph_vs_X
	  set no_graph_count 0
	  update
	  stop
	} else {		
	  while {$tSim >= [expr $t_final + $epsilon_t] && $running} {
	    pythag_evolve -$epsilon_t	;# Decrements t_sim by epsilon_t
	    draw_graph_vs_T
	    incr no_graph_count 1
	    if {$no_graph_count>=$graph_time_skip} {
	  	draw_graph_vs_X
	        LASSPentry_set .tmax [expr - $t_final + $tSim]
	        LASSPentry_set .tSim $tSim
		set no_graph_count 0
	  	update
	    }
	  }
	  if {($tSim > $t_final) && $running} {
	    pythag_evolve [expr $t_final - $tSim] ;# Remainder
          }
	  draw_graph_vs_T
	  LASSPentry_set .tmax [expr -$t_final + $tSim]
	  LASSPentry_set .tSim $tSim
	  draw_graph_vs_X
	  set no_graph_count 0
	  update
	  stop
	} 
    }
}

button .initialize -text "Initialize" -command {graph_initialize} 

pack .run .initialize -in .topbuttons -side right -fill x -expand yes -pady 2
pack .topbuttons -in .c -fill x -expand yes -pady 2

frame .user_name
    label .user_name.l -text "Your Name:"
    LASSPentry .user_name.e  {catch {set user_name [%W get]}}
    LASSPentry_set .user_name.e $user_name
    pack .user_name.l -side left
    pack .user_name.e -side left -fill x -expand yes
pack .user_name -in .c -fill x -expand yes -pady 2

frame .times
frame .tmaxf
label .tmaxf.l -text "Time to Run:"
LASSPentry .tmax \
    {expr {[catch {expr -[%W get]}] || [catch {set tmax [%W get]}] }} -width 5
pack .tmaxf.l -side left
pack .tmax -in .tmaxf -side left -fill x -expand yes
LASSPentry_set .tmax $tmax
pack .tmaxf -in .times -side right  -pady 2 

frame .tSimf
label .tSimf.l -text "Time:"
LASSPentry .tSim \
    {expr {[catch {expr -[%W get]}] || [catch {set tSim [%W get]}] }} -width 5
pack .tSimf.l -side left
pack .tSim -in .tSimf -side left -fill x -expand yes
LASSPentry_set .tSim $tSim
pack .tSimf -in .times -side left -pady 2 

pack .times -in .c

frame .bottombuttons
LASSPpresets .presets presetsave
button .config -text "Configure" -command {configure}
button .help -text "Work & Help" \
	-command {LASSPhelp2 .t2 [file join $SSS_DIR lib/pythag/pythag.htm]}
button .quit -text "Quit" -command {set running 0; exit 0}

frame .bottombuttons.tb
pack .presets .config -in .bottombuttons.tb -side left -fill x -expand yes
frame .bottombuttons.bb
pack .help .quit -in .bottombuttons.bb -side left -fill x -expand yes

pack .bottombuttons.tb .bottombuttons.bb -fill x -expand yes
pack .bottombuttons -in .c -fill x -expand yes

pack .c -side right



############################## special bindings #######################3
bind . <Key-i> {.initialize invoke}
bind . <Key-Escape> {.quit invoke}

wm minsize . 610 472

graph_initialize
.run invoke
