set SSS_PROG "schrdgr"


#################### Global variables linked to C ######################

# Debugging Tool
# LASSPinterp .interp	

# Schrdgr parameters

set Energy 0.
set Psi0 0.			;# Value of Psi at x=0
set PsiPrime0 1.		;# Value of PsiPrime at x=0
				;# Only changes normalization!

set Vbottom 0.			;# (eV) Bottom of square well vShelf
set Vshelf 3.			;# (eV) Shelf of vShelf
set Xshelf 1.0			;# (nm) Point where vShelf steps up
set Xmax $Xshelf		;# Maximum point to which to integrate

# Runge-Kutta Parameters

set delta_x 0.01		;# Step size for Runge Kutta 


# Other Parameters

set potential_type "InfSquare"	;# Initial potential infinite square
# Comment out for DEBUG test
Quantum_switch_potential	;# Tell C-code which potential
set graph_type  "Psi(X)"	;# Psi(X) or |Psi(X)|^2
set user_name "Waldo"		;# User's name
set running 0			;# Zero to stop animation, one while running 
set Escale 0.1			;# Shrink energy y-axis to make Psi big

################### Global variables Not linked to C ####################
# num_graphs - used for the LASSPgraph routines

set num_graphs 0
set max_steps 1000		;# Keep user from typing in too big a number,
				;# waiting forever

################################## center controls  ############

proc Shoot_Try {} {
    global running
    if {$running==0} {
	Shoot
	update idletasks
    } else { puts stdout "Already running" }
}


LASSPscale2 .energy -text "Energy (eV): " -scrollmin -0 -scrollmax 5 \
	-initial 0. -scrollinc 0.001 -type real \
	-var Energy -command {
	    global running
	    Shoot_Try
	}

frame .c


frame .potential_frame -relief raised -bd 2
    label .potential_frame.l -text " Potential"
    LASSPselection .potential_type {} potential_type {
	    Quantum_switch_potential;
	    if {$potential_type == "InfSquare"} {
		LASSPscale2_set .xmax $Xshelf
		LASSPscale2_configure .energy -scrollmin 0
		graph_label
		Shoot_Try
	    } elseif {$potential_type == "SquareShelf"} {
		LASSPscale2_set .xmax [expr 1.5*$Xshelf]
		LASSPscale2_configure .energy -scrollmin 0
		graph_label
		Shoot_Try
	    } elseif {$potential_type == "Hydrogen"} {
		LASSPscale2_set .xmax 0.5 
		LASSPscale2_configure .energy -scrollmin -20
		graph_label
		global running
		Shoot
	    }
	} "InfSquare" "SquareShelf" "Hydrogen"
# pack .potential_frame.l -side right
# pack .potential_type -in .potential_frame -side right -fill x -expand yes
pack .potential_frame.l -side right -fill x -expand yes
pack .potential_type -in .potential_frame -side right
frame .graph_frame -relief raised -bd 2
    label .graph_frame.l -text "Graph: "
    LASSPselection .graph_type {} graph_type {
	graph_label; 
	Shoot_Try
	} "Psi(X)" "|Psi(X)|^2"
# pack .graph_frame.l -side left
# pack .graph_type -in .graph_frame -side left -fill x -expand yes
pack .graph_frame.l -side left -fill x -expand yes
pack .graph_type -in .graph_frame -side left

# JPS modified?
#	-test {(%v>0)&&(%v<10)} 
LASSPscale2 .xmax -text "Distance to Shoot: (nm) " -scrollmin 0 -scrollmax 10 \
	-initial 1. -scrollinc 0.1 -type real -var Xmax \
	-test {{%v>0} && {%v<10}} \
	-command { Shoot_Try }

button .copygraph -text "Copy Graph" -command {
    incr num_graphs 1
    graph_create .graph$num_graphs .graph 
    .graph$num_graphs.graph configure -title $user_name
#    .graph$num_graphs.graph psconfigure -landscape true\
#	-pageheight 9i -pagewidth 6.5i
    .graph$num_graphs.graph postscript configure -landscape yes
}


frame .user_name -relief raised -bd 2 
    label .user_name.l -text "Your Name:"
    LASSPentry .user_name.e  {catch {set user_name [%W get]}}
    LASSPentry_set .user_name.e $user_name
    pack .user_name.l -side left
    pack .user_name.e -side left -fill x -expand yes

# LASSPpresets .presets presetsave
button .configure -text "Configure" -command {configure}
button .help -text "Assignment, and Help" \
    -command {LASSPhelp2 .t2 [file join $SSS_DIR lib/schrdgr/schrdgr.htm]}
button .quit -text "Quit" -command {exit}

pack .potential_frame -in .c -fill x -pady 2
pack .graph_frame -in .c -fill x -pady 2
pack .xmax -in .c -fill x -pady 2
pack .copygraph -in .c -fill x -pady 2
pack .user_name -in .c -fill x
pack .configure -in .c -fill x 
# pack .presets -in .c -fill x 
pack .help -in .c -fill x 
pack .quit -in .c -fill x 
wm minsize . 200 200

################# graph controls ##############################################
frame .g


################### the graph window ######################################

# JPS modified?
blt_graph .graph -width 300 -height 300 -relief raised -bd 2 \
	-plotbackground white

#graph .graph -width 300 -height 300 -relief raised -bd 2 \
#	-plotbackground white
.graph element create data
# JPS: modified?
.graph element configure data -fg blue -linewidth 4
# defaults to showing symbol, must set to null
#.graph element configure data -color blue -linewidth 4 -symbol ""
.graph element create potential
# JPS: modified?
.graph element configure potential -fg black -linewidth 1
# defaults to showing symbol, must set to null
#.graph element configure potential -color black -linewidth 1 -symbol ""
# newer BLT uses activeLine pen when a dataset is active
# need to change color and remove symbol 
# JPS: Configure pen twice, for data and potential? or only once?
#.graph pen configure "activeLine" -color pink -symbol ""
.graph legend configure -mapped no
# mapped (shown) goes to hide in later blt
#.graph legend configure -hide true
.graph yaxis configure -title ""
proc graph_label {} {
    global graph_type potential_type Escale
    if {$potential_type == "Hydrogen"} {
	if {$graph_type == "Psi(X)"} {
	    .graph xaxis configure -title \
		"X*Psi, and (V(X)-Energy) (eV*$Escale), vs X (nm)"
	} elseif {$graph_type == "|Psi(X)|^2"} {
	    .graph xaxis configure -title \
		"(X*|Psi|)^2, and (V(X)-Energy) (eV*$Escale), vs X (nm)"
	} else {puts stderr "Unknown graph type in graph_label"}
    } elseif {$potential_type == "InfSquare"} {
	if {$graph_type == "Psi(X)"} {
	    .graph xaxis configure -title \
		"Psi vs X (nm)"
	} elseif {$graph_type == "|Psi(X)|^2"} {
	    .graph xaxis configure -title \
		"|Psi|^2 (prob/nm) vs X (nm)"
	} else {puts stderr "Unknown graph type in graph_label"}
    } else {
	if {$graph_type == "Psi(X)"} {
	    .graph xaxis configure -title \
		"Psi, and (V(X)-Energy) (eV*$Escale), vs X (nm)"
	} elseif {$graph_type == "|Psi(X)|^2"} {
	    .graph xaxis configure -title \
		"|Psi|^2, and (V(X)-Energy) (eV*$Escale), vs X (nm)"
	} else {puts stderr "Unknown graph type in graph_label"}
    }

}
graph_label
# .graph xaxis configure -title \
#	"Psi vs X (nm), and (V(X)-Energy) (eV*$Escale) vs X (nm)"
LASSPgrzoom .graph
LASSPgrcursor .graph "format {%.4lf,   %.4lf} "


################## Placing the pieces ###################################
pack .energy -side top -fill x -pady 2
pack .graph -in .g -side top -fill both -expand yes 
pack .g -side right -fill both -expand yes -padx 5 -pady 5
pack .c -side right -fill y -padx 5 -pady 5
wm minsize . 200 200


################################## the presetsave procedure ############
proc presetsave {file} {
    #global variables
    foreach s {Psi0 PsiPrime0 Vbottom Vshelf Xshelf delta_x} {
        global $s
        puts $file "set $s [set $s]"
    }

    #selections
    foreach s {.potential_type} {
        set val [LASSPselection_set $s]
        puts $file "LASSPselection_set $s \"$val\""
    }

    #sliders
    foreach s {.energy .xmax} {
        set val [LASSPscale2_configure $s -scrollinc]
        puts $file "LASSPscale2_configure $s -scrollinc $val"
        set val [LASSPscale2_configure $s -scrollmin]
        puts $file "LASSPscale2_configure $s -scrollmin $val"
        set val [LASSPscale2_configure $s -scrollmax]
        puts $file "LASSPscale2_configure $s -scrollmax $val"
        set val [LASSPscale2_configure $s -scrolltype]
        puts $file "LASSPscale2_configure $s -scrolltype $val"
        set val [LASSPscale2_configure $s -font]
        puts $file "LASSPscale2_configure $s -font $val"

	set val [LASSPscale2_get $s]
        puts $file "LASSPscale2_set -nocommand $s $val"
    } 
    
}

################################## create a configure window ############
proc configure {} {

    global Escale Psi0 PsiPrime0
    global Vbottom Vshelf Xshelf Xmax
    global delta_x
    global potential_type

# Reset toplevel windows if they exist
    if {[winfo exists .schrdgr_configure]} {
    } else {
        toplevel .schrdgr_configure
        wm title .schrdgr_configure "schrdgr Configure"
    }

    set w .schrdgr_configure

    frame $w.controls
    label $w.controls.l -text "schrdgr Configure"
    button $w.controls.b -text Close -command "destroy $w"

    pack $w.controls.l -side left -fill x -expand yes
    pack $w.controls.b -side left -fill x -expand yes
    pack $w.controls -fill x -expand yes -pady 1

    frame $w.potentials
	label $w.potentials.l -text \
		"schrdgr Potential Parameters"
	pack $w.potentials.l -fill x -expand yes

	frame $w.potentials.vbottom
	label $w.potentials.vbottom.l -text "Potential at Bottom (eV):"
	LASSPentry $w.potentials.vbottom.e  {
		catch {set Vbottom [%W get]; Shoot_Try }}
	LASSPentry_set $w.potentials.vbottom.e  $Vbottom
	pack $w.potentials.vbottom.l -side left
	pack $w.potentials.vbottom.e -side left -fill x -expand yes
	pack $w.potentials.vbottom -fill x -expand yes -pady 1

	frame $w.potentials.vshelf
	label $w.potentials.vshelf.l -text "Potential at Shelf (eV):"
	LASSPentry $w.potentials.vshelf.e  {
		catch { set Vshelf [%W get]; Shoot_Try }}
	LASSPentry_set $w.potentials.vshelf.e  $Vshelf
	pack $w.potentials.vshelf.l -side left
	pack $w.potentials.vshelf.e -side left -fill x -expand yes
	pack $w.potentials.vshelf -fill x -expand yes -pady 1

	frame $w.potentials.xshelf
	label $w.potentials.xshelf.l -text "Position of Shelf (nm):"
	LASSPentry $w.potentials.xshelf.e  {
		catch { set Xshelf [%W get]; Shoot_Try }}
	LASSPentry_set $w.potentials.xshelf.e  $Xshelf
	pack $w.potentials.xshelf.l -side left
	pack $w.potentials.xshelf.e -side left -fill x -expand yes
	pack $w.potentials.xshelf -fill x -expand yes -pady 1

	frame $w.potentials.escale
	label $w.potentials.escale.l -text "Shrink energy y-axis by:"
	LASSPentry $w.potentials.escale.e  {
		catch {
		    set Escale [%W get];
		    .graph xaxis configure -title \
	    "Psi vs X (nm), and (V(X)-Energy) (eV*$Escale) vs X (nm)"
		    Shoot_Try
		}}
	LASSPentry_set $w.potentials.escale.e  $Escale
	pack $w.potentials.escale.l -side left
	pack $w.potentials.escale.e -side left -fill x -expand yes
	pack $w.potentials.escale -fill x -expand yes -pady 1

     pack $w.potentials -fill x -expand yes -pady 1

    frame $w.initial_conditions
        label $w.initial_conditions.l -text "Initial Conditions"
        pack $w.initial_conditions.l -fill x -expand yes

        frame $w.initial_conditions.psi0
        label $w.initial_conditions.psi0.l -text "Initial Psi at x=0:"
        LASSPentry $w.initial_conditions.psi0.e  {
		catch { set Psi0 [%W get]; Shoot_Try }}
        LASSPentry_set $w.initial_conditions.psi0.e $Psi0
        pack $w.initial_conditions.psi0.l -side left
        pack $w.initial_conditions.psi0.e -side left -fill x -expand yes
        pack $w.initial_conditions.psi0 -fill x -expand yes -pady 1

       frame $w.initial_conditions.psiPrime0
        label $w.initial_conditions.psiPrime0.l \
		 -text "Initial PsiPrime (dPsi/dx):"
        LASSPentry $w.initial_conditions.psiPrime0.e  {
		catch { set PsiPrime0 [%W get]; Shoot_Try }}
        LASSPentry_set $w.initial_conditions.psiPrime0.e $PsiPrime0
        pack $w.initial_conditions.psiPrime0.l -side left
        pack $w.initial_conditions.psiPrime0.e -side left -fill x -expand yes
        pack $w.initial_conditions.psiPrime0 -fill x -expand yes -pady 1
 
    pack $w.initial_conditions -fill x -expand yes -pady 1

    frame $w.algorithm
        label $w.algorithm.l -text "Algorithm"
        pack $w.algorithm.l -fill x -expand yes

        frame $w.algorithm.delta_x
        label $w.algorithm.delta_x.l \
                -text "Spatial step delta_x(nm): (smaller: more accurate)\n"
        LASSPentry $w.algorithm.delta_x.e {
		catch { set delta_x [%W get]; Shoot_Try }}
        LASSPentry_set $w.algorithm.delta_x.e $delta_x
        pack $w.algorithm.delta_x.l -side left
        pack $w.algorithm.delta_x.e \
                -side left -fill x -expand yes
        pack $w.algorithm.delta_x \
                 -fill x -expand yes -pady 1

    pack $w.algorithm -fill x -expand yes -pady 1

    label $w.warning -text "Close and Reopen to See Changes from Within Program"
    pack $w.warning -fill x -expand yes

}



############################## special bindings ########################
# bind . <Key-i> {.init invoke}
bind . <Key-Escape> {.quit invoke}

update
# Comment out for DEBUG test
Shoot_Try
